#!/usr/bin/perl

package Text::Lists;

use strict;
use Carp;
#dev# use Data::Dumper;

$Text::Lists::VERSION = qw$Revision: 0.1.1$[1];

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {
		-irs => '-',  # input range separator
		-ors => '-',  # output range separator
		-ils => ' ',  # input list separator
		-ols => ' ',  # output list separator
	};
    # build a set out of valid parameter names:
    my %params = map { $_ => 1 } qw( -irs -ils -ors -ols -list -string);

    # import valid parameters and their values
    my %h = @_;
    for my $k ( keys %h ) {
        #dev# print $k," ",$h{$k},"\n";
        if ( defined $params{$k} ) {
            $self->{$k} = $h{$k};
        } else {
            Carp::confess "invalid parameter '$k'";
        }
    }
    # ...

    bless ($self,$class);

	if ( $self->{-list} ) {
		$self->{clist} = $self->condenseList;
	} elsif ( $self->{-string} ) {
		$self->{clist} = $self->importString;
	}

    return $self;
}

sub importString () {
	my $self = shift;

	$self->{-string} =~ s/(^\s+|\s+$)//g;

	my $l = [ split $self->{-ils},$self->{-string} ];
	$self->{-list} = $l;
	#dev# print "-l: ",Dumper($self->{-list});
	$self->{clist} = $self->condenseList;
	return $self->{clist};
}

sub condenseList () {
	my $self = shift;

	my @List = sort { $a <=> $b } @{$self->{-list}};
	#dev# print "condenseList: @List\n";
	my @List2 = ();

	my $last = undef;
	my $lastlow = undef;
	my $lasthigh = undef;

	for my $i ( @List,undef ) { # process last element also
		if ( ! defined $last ) {
			#dev# print "! $i\n";
			$lastlow  = $i;
		} else { # defined $last
			#dev# print "L $i $last\n";
			if ( $i == $last+1 ) {
				$lasthigh = $i;
			} else {
				if ( $lastlow == $lasthigh ) {
					push @List2, $lastlow;
				} else {
					push @List2, $lastlow."-".$lasthigh;
				}
				$lastlow  = $i;
			}
		}
		$lasthigh = $i;
		$last     = $i;
	}
	#dev# print "condenseList: @List2\n";

	if ( wantarray ) {
		return @List2;
	} else {
		return join " ",@List2;
	}
}

sub expandList () {
	my $self = shift;

	my @List = sort { $a <=> $b } @{$self->{-list}};
	my @List2 = ();

	for my $i ( @List ) {
		if ( $i =~ m/^(\d+)-(\d+)$/ ) {
			push @List2, ($1..$2);
		} else {
			push @List2, $i;
		}
	}

	if ( wantarray ) {
		return @List2;
	} else {
		return join " ",@List2;
	}
}

sub condensedString () {
	my $self = shift;

	return join $self->{-ols}, $self->expandList;
}

1;
